var posAZ = 50000;         // Imaginary azimuth value, corrected from * .ini file
var dirAZ = false;         // Azimuth motor direction, relay off
var posEL = 50000;         // Imaginary elevation value, corrected from * .ini file
var dirEL = false;         // Elevation motor direction, relay off
var orgAZ = true;          // Azimuth origin switch, Imaginary value
var orgEL = true;          // Elevation origin switch, Imaginary value
var SatName = "Default";   // The name of the current satellite
var MemSaveActive = false; // Do I need to save a new preset value?

var Gpio = require('onoff').Gpio; //require onoff to control GPIO

var AZonoff = new Gpio(23, 'out'); //declare GPIO23 an output, AZ motor On/Off
var ELonoff = new Gpio(27, 'out'); //declare GPIO27 an output, EL motor On/Off
var AZdirection = new Gpio(4, 'out'); //declare GPIO4 an output, AZ direction
var ELdirection = new Gpio(17, 'out'); //declare GPIO17 an output, EL direction

var AZphaseA = new Gpio(20, 'in', 'rising'); //declare GPIO20 an input, AZ phase A
var AZphaseB = new Gpio(21, 'in', 'rising'); //declare GPIO21 an input, AZ phase B
var ELphaseB = new Gpio(26, 'in', 'rising'); //declare GPIO26 an input, EL phase B
var ELphaseA = new Gpio(19, 'in', 'rising'); //declare GPIO19 an input, EL phase A
var AZoriginS = new Gpio(12, 'in', 'both'); //declare GPIO12 an input, AZ origin switch
var ELoriginS = new Gpio(16, 'in', 'both'); //declare GPIO16 an input, EL origin switch

var fs = require('fs');    //require filesystem to read html files
var ini = require('ini');  // file for saving configuration and preferences
var url = require('url');  // parsing query

AZphaseA.watch(function (err, value) {  // Interrupt operation 
   if (err) { //if an error
     console.error('There was an error AZphaseA', err); //output error message to console
     return;
     }
   if (dirAZ) {
     posAZ --; // respond to an interruption
     }
     else {
     posAZ ++; // respond to an interruption
     }
   });

AZphaseB.watch(function (err, value) {  // Interrupt operation
   if (err) { //if an error
    console.error('There was an error AZphaseB', err); //output error message to console
  return;
  }
  if (dirAZ) {
   posAZ --; // respond to an interruption
   }
   else {
   posAZ ++; // respond to an interruption
   }
});

ELphaseB.watch(function (err, value) {  // Interrupt operation
   if (err) { //if an error
    console.error('There was an error ELphaseB', err); //output error message to console
  return;
  }
   if (dirEL) {
   posEL --; // respond to an interruption
   }
   else {
   posEL ++; // respond to an interruption
   }
});

ELphaseA.watch(function (err, value) {  // Interrupt operation
   if (err) { //if an error
    console.error('There was an error ELphaseA', err); //output error message to console
  return;
  }
   if (dirEL) {
   posEL --; // respond to an interruption
   }
   else {
   posEL ++; // respond to an interruption
   }
});

AZoriginS.watch(function (err, value) {  // Interrupt operation
   if (err) { //if an error
    console.error('There was an error AZoriginS', err); //output error message to console
  return;
  }
   if (value) {
   orgAZ=true;  // Switch open, true azimuth zero point
   posAZ=0;     // Set the actual default azimuth value    
   }
   else {
   orgAZ=false; // Switch closing
   }
});

ELoriginS.watch(function (err, value) {  // Interrupt operation
   if (err) { //if an error
    console.error('There was an error ELoriginS', err); //output error message to console
  return;
  }
   if (value) {
   orgEL=true;  // Switch opening, true zero elevation point
   posEL=0;     // Set the actual default elevation value   
   }
   else {
   orgEL=false; // Switch closing
   }
});

function unexportOnClose() { //function to run when exiting program
  AZonoff.writeSync(0);      // Stop the engine AZ
  AZdirection.writeSync(0);  // Turns off the AZ direction relay
  ELonoff.writeSync(0);      // Stop the engine EL
  ELdirection.writeSync(0);  // Turns off the EL direction relay
  AZonoff.unexport();        // Unexport AZonoff to free resources
  AZdirection.unexport();    // Unexport AZdirection to free resources
  ELonoff.unexport();        // Unexport ELonoff to free resources
  ELdirection.unexport();    // Unexport ELdirection to free resources
  ELphaseA.unexport();       // Unexport ELphaseA to free resources
  ELphaseB.unexport();       // Unexport ELphaseB to free resources
  AZphaseB.unexport();       // Unexport AZphaseB to free resources
  AZphaseA.unexport();       // Unexport AZphaseA to free resources
  AZoriginS.unexport();      // Unexport AZoriginS to free resources
  ELoriginS.unexport();      // Unexport ELoriginS to free resources
  config.Default.PosAZ = posAZ;
  config.Default.PosEL = posEL;
  config.Default.Name = SatName;
  fs.writeFileSync('./config.ini', ini.stringify(config)); // save current position
  process.exit();            // end of program, on ctrl c
};

process.on('SIGINT', unexportOnClose); // Activates an interrupt from the value of the input pins

// initialization of configuration and preferences
var config = ini.parse(fs.readFileSync('./config.ini', 'utf-8'));
posAZ = config.Default.PosAZ;
newposAZ = posAZ;
posEL = config.Default.PosEL;
newposEL = posEL;
SatName = config.Default.Name;
console.log(posAZ + " ; " + posEL + " ; " + SatName); 

var http = require('http').createServer(function handler(req, res) { //create server

var q = url.parse(req.url, true); // go through and process the html query from the browser
  var  filename = "." + q.pathname;
  if (filename =="./") {filename = "./index.html"};
  var contentTypesByExtension = {
    '.html': "text/html",
    '.css':  "text/css",
    '.js':   "text/javascript"
    };
  fs.readFile(filename,  function(err, data) {
    if (err) { //if an error
       res.writeHead(404, {'Content-Type': 'text/html'}); //send error page
       return res.end("404 Not Found");
    }  
    res.writeHead(200);
    res.write(data); //write data from index.html
    return res.end();
  });
});

var io = require('socket.io')(http) //require socket.io module and pass the http object

http.listen(8080); // listen to port 8080; Here you can set the default port 80

io.sockets.on('connection', function (socket) { // WebSocket Connection
  var buttonState1 = 0; //variable to store button data1
  var buttonState2 = 0; //variable to store button data2

function movetonewpos(newposAZ, newposEL) {  // turn the dish to the new position
let stopAZ = false;
let stopEL = false;
// Case 1 from 8
if ((newposAZ < posAZ) && (newposEL == posEL)) { 
  dirAZ = true;
  AZdirection.writeSync(1);  // Turn on relay AZ direction
  AZonoff.writeSync(1); // Start the AZ engine
  const mojeid = setInterval(function() {
    if (newposAZ > posAZ) {
      AZonoff.writeSync(0); // Stop the engine AZ
      dirAZ = false;
      AZdirection.writeSync(0); // Turn off relay AZ direction 
      setTimeout(() => {
            // runs after 0,2 seconds, setrvacnost motoru
            socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
            config.Default.PosAZ = posAZ;
            config.Default.PosEL = posEL;
            config.Default.Name = SatName;
            fs.writeFileSync('./config.ini', ini.stringify(config));
            }, 200);       
      clearInterval(mojeid);
      }
    }, 5);  // every 5 mS
  }
//  Case 2 from 8
if ((newposAZ > posAZ) && (newposEL == posEL)) { 
  AZonoff.writeSync(1); // Start the AZ engine
  const mojeid = setInterval(function() {
    if (newposAZ < posAZ) {
      AZonoff.writeSync(0); // Stop the engine AZ
      setTimeout(() => {
            // runs after 0,2 seconds, setrvacnost motoru
            socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
            config.Default.PosAZ = posAZ;
            config.Default.PosEL = posEL;
            config.Default.Name = SatName;
            fs.writeFileSync('./config.ini', ini.stringify(config));
            }, 200);       
      clearInterval(mojeid);
      }
    }, 5);  // every 5 mS
  }
// Case 3 from 8
if ((newposEL < posEL) && (newposAZ == posAZ)) { 
  dirEL = true;
  ELdirection.writeSync(1);  // Turn on relay EL direction
  ELonoff.writeSync(1); // Start the EL engine
  const mojeid = setInterval(function() {
    if (newposEL > posEL) {
      ELonoff.writeSync(0); // Stop the engine EL
      dirEL = false;
      ELdirection.writeSync(0); // Turn off relay EL direction 
      setTimeout(() => {
            // runs after 0,2 seconds, setrvacnost motoru
            socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
            config.Default.PosAZ = posAZ;
            config.Default.PosEL = posEL;
            config.Default.Name = SatName;
            fs.writeFileSync('./config.ini', ini.stringify(config));
            }, 200);       
      clearInterval(mojeid);
      }
    }, 5);  // every 5 mS
  }
//  Case 4 from 8
if ((newposAZ < posAZ) && (newposEL < posEL)) {
  dirAZ = true;
  AZdirection.writeSync(1);  // Turn on relay AZ direction
  AZonoff.writeSync(1); // Start the AZ engine
  dirEL = true;
  ELdirection.writeSync(1);  // Turn on relay EL direction
  ELonoff.writeSync(1); // Start the EL engine
  const mojeid = setInterval(function() {
    if (newposAZ > posAZ) {
      AZonoff.writeSync(0); // Stop the engine AZ
      dirAZ = false;
      AZdirection.writeSync(0); // Turn off relay AZ direction
      stopAZ = true;
      }
    if (newposEL > posEL) {
      ELonoff.writeSync(0); // Stop the engine EL
      dirEL = false;
      ELdirection.writeSync(0); // Turn off relay EL direction
      stopEL = true;
      }
    if (stopAZ && stopEL ) {  
      setTimeout(() => {
      // runs after 0,2 seconds, setrvacnost motoru
      socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
      config.Default.PosAZ = posAZ;
      config.Default.PosEL = posEL;
      config.Default.Name = SatName;
      fs.writeFileSync('./config.ini', ini.stringify(config));
      }, 200);
      stopAZ = false;
      stopEL = false;       
      clearInterval(mojeid);
      }
    }, 5);  // every 5 mS
  }
//  Case 5 from 8
if ((newposAZ > posAZ) && (newposEL < posEL)) {
  AZonoff.writeSync(1); // Start the AZ engine
  dirEL = true;
  ELdirection.writeSync(1);  // Turn on relay EL direction
  ELonoff.writeSync(1); // Start the EL engine
  const mojeid = setInterval(function() {
    if (newposAZ < posAZ) {
      AZonoff.writeSync(0); // Stop the engine AZ
      stopAZ = true;
      }
    if (newposEL > posEL) {
      ELonoff.writeSync(0); // Stop the engine EL
      dirEL = false;
      ELdirection.writeSync(0); // Turn off relay EL direction
      stopEL = true;
      }
    if (stopAZ && stopEL ) {  
      setTimeout(() => {
      // runs after 0,2 seconds, setrvacnost motoru
      socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
      config.Default.PosAZ = posAZ;
      config.Default.PosEL = posEL;
      config.Default.Name = SatName;
      fs.writeFileSync('./config.ini', ini.stringify(config));
      }, 200);
      stopAZ = false;
      stopEL = false;       
      clearInterval(mojeid);
      }
    }, 5);  // every 5 mS
  }               
//  Case 6 from 8
if ((newposEL > posEL) && (newposAZ == posAZ)) { 
  ELonoff.writeSync(1); // Start the EL engine
  const mojeid = setInterval(function() {
    if (newposEL < posEL) {
      ELonoff.writeSync(0); // Stop the engine EL
      setTimeout(() => {
            // runs after 0,2 seconds, setrvacnost motoru
            socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
            config.Default.PosAZ = posAZ;
            config.Default.PosEL = posEL;
            config.Default.Name = SatName;
            fs.writeFileSync('./config.ini', ini.stringify(config));
            }, 200);       
      clearInterval(mojeid);
      }
    }, 5);  // every 5 mS
  } 
//  Case 7 from 8
if ((newposAZ < posAZ) && (newposEL > posEL)) {
  dirAZ = true;
  AZdirection.writeSync(1);  // Turn on relay AZ direction
  AZonoff.writeSync(1); // Start the AZ engine
  ELonoff.writeSync(1); // Start the EL engine
  const mojeid = setInterval(function() {
    if (newposAZ > posAZ) {
      AZonoff.writeSync(0); // Stop the engine AZ
      dirAZ = false;
      AZdirection.writeSync(0); // Turn off relay AZ direction
      stopAZ = true;
      }
    if (newposEL < posEL) {
      ELonoff.writeSync(0); // Stop the engine EL
      stopEL = true;
      }
    if (stopAZ && stopEL ) {  
      setTimeout(() => {
      // runs after 0,2 seconds, setrvacnost motoru
      socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
      config.Default.PosAZ = posAZ;
      config.Default.PosEL = posEL;
      config.Default.Name = SatName;
      fs.writeFileSync('./config.ini', ini.stringify(config));
      }, 200);
      stopAZ = false;
      stopEL = false;       
      clearInterval(mojeid);
      }
    }, 5);  // every 5 mS
  }           
//  Case 8 from 8
if ((newposAZ > posAZ) && (newposEL > posEL)) { 
  AZonoff.writeSync(1); // Start the AZ engine
  ELonoff.writeSync(1); // Start the EL engine
  const mojeid = setInterval(function() {
    if (newposAZ < posAZ) {
      AZonoff.writeSync(0); // Stop the engine AZ
      stopAZ = true;
      }
    if (newposEL < posEL) {
      ELonoff.writeSync(0); // Stop the engine EL
      stopEL = true;
      }
    if (stopAZ && stopEL ) {  
      setTimeout(() => {
      // runs after 0,2 seconds, setrvacnost motoru
      socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
      config.Default.PosAZ = posAZ;
      config.Default.PosEL = posEL;
      config.Default.Name = SatName;
      fs.writeFileSync('./config.ini', ini.stringify(config));
      }, 200);
      stopAZ = false;
      stopEL = false;       
      clearInterval(mojeid);
      }
    }, 5);  // every 5 mS
  }     
// end of function movetonewpos   
};
  
  socket.on('state', function (data1,data2) { //get button state from client
  buttonState1 = data1;
  buttonState2 = data2;
  switch (buttonState2) {
    case 10: 
        if (buttonState1 != AZonoff.readSync()) { // The AZ+ button has been pressed
        dirAZ = false;
          if (buttonState1 == 1) {
            AZonoff.writeSync(1); // Start the engine AZ
            }
            else
            {
            AZonoff.writeSync(0); // Stop the engine AZ
            SatName = "Not Set";
            setTimeout(() => {
            // runs after 0,1 seconds, motor inertia
            socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
            }, 100);
            config.Default.PosAZ = posAZ;
            config.Default.PosEL = posEL;
            config.Default.Name = SatName;
            fs.writeFileSync('./config.ini', ini.stringify(config));
            }
          }
        break;
    case 11:
        if (buttonState1 != AZonoff.readSync()) { // The AZ- button has been pressed
        dirAZ = true;
          if (buttonState1 == 1) {
            AZdirection.writeSync(1); // Activate relay AZ direction
            AZonoff.writeSync(1);     // Start the engine AZ
            }
            else
            {
            AZonoff.writeSync(0); // Stop the engine AZ
            AZdirection.writeSync(0); // Turns off the AZ direction relay
            SatName = "Not Set";
            setTimeout(() => {
            // runs after 0,1 seconds, motor inertia
            socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
            }, 100);
            config.Default.PosAZ = posAZ;
            config.Default.PosEL = posEL;
            config.Default.Name = SatName;
            fs.writeFileSync('./config.ini', ini.stringify(config));
            }
          }  
        break;          
    case 12: 
        if (buttonState1 != ELonoff.readSync()) { // The EL+ button has been pressed
        dirEL = false;
          if (buttonState1 == 1) {
            ELonoff.writeSync(1); // Start the engine EL
            }
            else
            {
            ELonoff.writeSync(0); // Stop the engine EL
            SatName = "Not Set";
            setTimeout(() => {
            // runs after 0,1 seconds, motor inertia
            socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
            }, 100);
            config.Default.PosAZ = posAZ;
            config.Default.PosEL = posEL;
            config.Default.Name = SatName;
            fs.writeFileSync('./config.ini', ini.stringify(config));
            }
          }
        break;
    case 13: 
        if (buttonState1 != ELonoff.readSync()) { // The EL- button has been pressed
        dirEL = true;
          if (buttonState1 == 1) {
            ELdirection.writeSync(1); // Activate relay EL direction
            ELonoff.writeSync(1); // Start the engine EL
            }
            else
            {
            ELonoff.writeSync(0); // Stop the engine EL
            ELdirection.writeSync(0); // Turns off the EL direction relay
            SatName = "Not Set";
            setTimeout(() => {
            // runs after 0,1 seconds, motor inertia
            socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); // Client feedback
            }, 100);
            config.Default.PosAZ = posAZ;
            config.Default.PosEL = posEL;
            config.Default.Name = SatName;
            fs.writeFileSync('./config.ini', ini.stringify(config));
            }
          }  
        break;
    }
  });
  
  socket.on('memory', function (data) { //get button state from client
  buttonState1 = data;
  //buttonState2 = data2;
  switch (buttonState1) {
    case 0:                       // Client feedback the current position
      socket.emit("backpos", {AZ: posAZ, EL: posEL, Name: SatName}); 
      break;
    case 1:  // if the default value is nonzero
      if ((config.mem01.PosAZ != 0 ) && (config.mem01.PosEL != 0 )) {
        SatName = config.mem01.Name;
        movetonewpos(config.mem01.PosAZ, config.mem01.PosEL);
        } 
      break;
    case 2:
      if ((config.mem02.PosAZ != 0 ) && (config.mem02.PosEL != 0 )) {
        SatName = config.mem02.Name;
        movetonewpos(config.mem02.PosAZ, config.mem02.PosEL);
        }  
      break;          
    case 3:
      if ((config.mem03.PosAZ != 0 ) && (config.mem03.PosEL != 0 )) {
        SatName = config.mem03.Name; 
        movetonewpos(config.mem03.PosAZ, config.mem03.PosEL);
        }  
      break;
    case 4:
      if ((config.mem04.PosAZ != 0 ) && (config.mem04.PosEL != 0 )) {
        SatName = config.mem04.Name; 
        movetonewpos(config.mem04.PosAZ, config.mem04.PosEL);
        }  
      break;        
    case 5: 
      if ((config.mem05.PosAZ != 0 ) && (config.mem05.PosEL != 0 )) {
        SatName = config.mem05.Name;
        movetonewpos(config.mem05.PosAZ, config.mem05.PosEL);
        }  
      break; 
    case 6: 
      if ((config.mem06.PosAZ != 0 ) && (config.mem06.PosEL != 0 )) {
        SatName = config.mem06.Name;
        movetonewpos(config.mem06.PosAZ, config.mem06.PosEL);
        }  
      break;
    case 7:
      if ((config.mem07.PosAZ != 0 ) && (config.mem07.PosEL != 0 )) {
        SatName = config.mem07.Name; 
        movetonewpos(config.mem07.PosAZ, config.mem07.PosEL);
        }
      break; 
    case 8:
      if ((config.mem08.PosAZ != 0 ) && (config.mem08.PosEL != 0 )) { 
        SatName = config.mem08.Name;
        movetonewpos(config.mem08.PosAZ, config.mem08.PosEL);
        }
      break;
    case 9:
      if ((config.mem09.PosAZ != 0 ) && (config.mem09.PosEL != 0 )) {
        SatName = config.mem09.Name; 
        movetonewpos(config.mem09.PosAZ, config.mem09.PosEL);
        }
      break;                                      
    case 10:
      if ((config.mem10.PosAZ != 0 ) && (config.mem10.PosEL != 0 )) {
        SatName = config.mem10.Name; 
        movetonewpos(config.mem10.PosAZ, config.mem10.PosEL);
        }
      break;
    case 11:
      if ((config.mem11.PosAZ != 0 ) && (config.mem11.PosEL != 0 )) {
        SatName = config.mem11.Name; 
        movetonewpos(config.mem11.PosAZ, config.mem11.PosEL);
        }
      break;
    case 12:
      if ((config.mem12.PosAZ != 0 ) && (config.mem12.PosEL != 0 )) {
        SatName = config.mem12.Name;
        movetonewpos(config.mem12.PosAZ, config.mem12.PosEL);
        }
      break;          
    case 13:
      if ((config.mem13.PosAZ != 0 ) && (config.mem13.PosEL != 0 )) {
        SatName = config.mem13.Name; 
        movetonewpos(config.mem13.PosAZ, config.mem13.PosEL);
        }
      break;
    case 14: 
      if ((config.mem14.PosAZ != 0 ) && (config.mem14.PosEL != 0 )) {
        SatName = config.mem14.Name;
        movetonewpos(config.mem14.PosAZ, config.mem14.PosEL);
        }
      break;        
    case 15:
      if ((config.mem15.PosAZ != 0 ) && (config.mem15.PosEL != 0 )) {
        SatName = config.mem15.Name; 
        movetonewpos(config.mem15.PosAZ, config.mem15.PosEL);
        }
      break; 
    case 16:
      if ((config.mem16.PosAZ != 0 ) && (config.mem16.PosEL != 0 )) {
        SatName = config.mem16.Name; 
        movetonewpos(config.mem16.PosAZ, config.mem16.PosEL);
        }
      break;
    case 17: 
      if ((config.mem17.PosAZ != 0 ) && (config.mem17.PosEL != 0 )) {
        SatName = config.mem17.Name;
        movetonewpos(config.mem17.PosAZ, config.mem17.PosEL);
        }
      break; 
    case 18: 
      if ((config.mem18.PosAZ != 0 ) && (config.mem18.PosEL != 0 )) {
        SatName = config.mem18.Name;
        movetonewpos(config.mem18.PosAZ, config.mem18.PosEL);
        }
      break;
    case 19: 
      if ((config.mem19.PosAZ != 0 ) && (config.mem19.PosEL != 0 )) {
        SatName = config.mem19.Name;
        movetonewpos(config.mem19.PosAZ, config.mem19.PosEL);
        }
      break;                                      
    case 20: 
      if ((config.mem20.PosAZ != 0 ) && (config.mem20.PosEL != 0 )) {
        SatName = config.mem20.Name;
        movetonewpos(config.mem20.PosAZ, config.mem20.PosEL);
        }
      break;    
    }
  });
});
